@@ -5,7 +5,7 @@ import json
 
 from .read import read_ipynb
 from .test import assert_no_errors
-from .utils import BLUE, CYAN, GREEN, LIGHTPURPLE, NC, RED, \
+from .utils import BLUE, CYAN, GREEN, LIGHTPURPLE, YELLOW, NC, RED, \
     print_cropped_output
 
 
@@ -64,34 +64,13 @@ def compare_ipynb_notebooks(notebook_filenames, debug):
                 )
             comparison_errors.extend(source_errors)
 
-            len_0_output = len(cells[0]["outputs"])
-            len_1_output = len(cells[1]["outputs"])
+            # Ignore output containing warnings
+            remove_warning_ouput(cells[0]["outputs"],
+                                 cells[1]["outputs"],
+                                 debug)
 
             # Empty output is not necessarily bad, print it
-            (len_0_output == 0 or len_1_output == 0) and \
-                print(BLUE, "-".center(80, "-"), NC)
-            len_0_output == 0 and print(CYAN, "cells[0][outputs]: ", NC) \
-                and print_cropped_output(cells[0]["outputs"])
-            len_1_output == 0 and print(CYAN, "cells[1][outputs]: ", NC) \
-                and print_cropped_output(cells[1]["outputs"])
-            (len_0_output == 0 or len_1_output == 0) and \
-                print(BLUE, "-".center(80, "-"), NC)
-
-            # # NOTE: Commented out this seciton 2021-05-26, need to
-            #         think more about it...
-            # # Check if size of the output arrays are different
-            # if len_0_output != len_1_output:
-            #     message = "output sizes not the same"
-            #     print(RED, message, NC)
-            #     print(len_0_output)
-            #     for output in cells[0]["outputs"]:
-            #         print("  ", output)
-            #     print(len_1_output)
-            #     for output in cells[1]["outputs"]:
-            #         print("  ", output)
-            #     logging.error(message)
-            #     comparison_errors.append(cells[0]["outputs"])
-            #     comparison_errors.append(cells[1]["outputs"])
+            print_empty_output(cells)
 
             # Loop over outputs - can have multiple for each source cell
             for outputs in zip(cells[0]["outputs"], cells[1]["outputs"]):
@@ -138,6 +117,57 @@ def compare_ipynb_notebooks(notebook_filenames, debug):
     return comparison_result, error_message, error_content
 
 
+def print_empty_output(cells):
+    len_0_output = len(cells[0]["outputs"])
+    len_1_output = len(cells[1]["outputs"])
+
+    (len_0_output == 0 or len_1_output == 0) and \
+        print(BLUE, "-".center(80, "-"), NC)
+    len_0_output == 0 and print(CYAN, "cells[0][outputs]: ", NC) \
+        and print_cropped_output(cells[0]["outputs"])
+    len_1_output == 0 and print(CYAN, "cells[1][outputs]: ", NC) \
+        and print_cropped_output(cells[1]["outputs"])
+    (len_0_output == 0 or len_1_output == 0) and \
+        print(BLUE, "-".center(80, "-"), NC)
+
+
+def remove_warning_ouput(cells_0_outputs, cells_1_outputs, debug):
+
+    debug and print(GREEN, "** Entering remove_warning_ouput() **\n", NC)
+
+    len0 = len(cells_0_outputs)
+    len1 = len(cells_1_outputs)
+    debug and print("len(cells_0_outputs): ", len0)
+    debug and print("len(cells_1_outputs): ", len1)
+
+    # Different warning that may appear in output, can vary depending on
+    # versions, etc.
+    warnings_to_ignore = {}
+    warnings_to_ignore["text"] = [
+        "DeprecationWarning",
+        "History will not be written to the database",
+    ]
+
+    # Look for the warning output and remove it from further consideration
+    for outputs in [cells_0_outputs, cells_1_outputs]:
+        for index, output in enumerate(outputs):
+            for key in warnings_to_ignore:
+                if key in output:
+                    for value in warnings_to_ignore[key]:
+                        if value in output[key]:
+                            debug and print(CYAN, "ignore warning", index,
+                                            value, YELLOW, "\n   ",
+                                            output[key], NC)
+                            outputs.pop(index)
+
+    len0 = len(cells_0_outputs)
+    len1 = len(cells_1_outputs)
+    debug and print("len(cells_0_outputs): ", len0)
+    debug and print("len(cells_1_outputs): ", len1)
+
+    debug and print(GREEN, "** Leaving  remove_warning_ouput() **\n", NC)
+
+
 def check_for_errors(comparison_errors):
 
     comparison_result = False
@@ -271,13 +301,14 @@ def examine_output_diff(ignore_output_differences, nb0, nb1, debug):
 
         debug and print(BLUE, "  key:", NC, key)
 
-        # Check if the values are the same for the same key in the two
-        # dictionaries
+        # First check that the second dictionary has the same key
         if key not in nb1:
             comparison_pass = False
             debug and print(RED, "comparison_pass: ", comparison_pass, NC,
                             RED, "\n    key ", key, "not in nb1", NC)
 
+        # Check if the values are the same for the same key in the two
+        # dictionaries
         else:
             # The easy case - values are exactly the same, nothing more to do
             if nb0[key] == nb1[key]:
@@ -335,6 +366,11 @@ def examine_output_key_data(key, nb0, nb1, debug):
 
     for subkey in nb0[key]:
 
+        if subkey not in nb1[key]:
+            debug and print(BLUE, "    --> subkey:", NC, subkey,
+                            RED, "\n      not present in ", nb1[key], NC)
+            return False
+
         if nb0[key][subkey] == nb1[key][subkey]:
             debug and print(BLUE, "    --> subkey:", NC, subkey,
                             "\n", GREEN, "      same subkey values", NC)
